@@ -1,10 +1,10 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
-open Location
+open Cduce_loc
 type type_fun = Types.t -> bool -> Types.t
 
 val register: 
