@@ -1,10 +1,10 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
-open Location
+open Cduce_loc
 open Ident
 
 let () = Stats.gettimeofday := Unix.gettimeofday
@@ -85,8 +85,8 @@ let directive_help ppf =
 
 let rec print_exn ppf = function
   | Location (loc, w, exn) ->
-      Location.print_loc ppf (loc,w);
-      Location.html_hilight (loc,w); 
+      Cduce_loc.print_loc ppf (loc,w);
+      Cduce_loc.html_hilight (loc,w); 
       print_exn ppf exn
   | Value.CDuceExn v ->
       Format.fprintf ppf "Uncaught CDuce exception: @[%a@]@."
@@ -127,9 +127,9 @@ let rec print_exn ppf = function
 	U.print (Librarian.name cu)
 	Ident.print x
   | Ulexer.Error (i,j,s) ->
-      let loc = Location.loc_of_pos (i,j), `Full in
-      Location.print_loc ppf loc;
-      Location.html_hilight loc;
+      let loc = Cduce_loc.loc_of_pos (i,j), `Full in
+      Cduce_loc.print_loc ppf loc;
+      Cduce_loc.html_hilight loc;
       Format.fprintf ppf "%s" s
   | Parser.Error s | Stream.Error s -> 
       Format.fprintf ppf "Parsing error: %a@." print_protect s
@@ -151,7 +151,7 @@ let rec print_exn ppf = function
       Format.fprintf ppf "Invalid object file %s@." f
   | Librarian.CannotOpen f ->
       Format.fprintf ppf "Cannot open file %s@." f
-  | Location.Generic s ->
+  | Cduce_loc.Generic s ->
       Format.fprintf ppf "%a@." print_protect s
   | Ns.Label.Not_unique ((ns1,s1),(ns2,s2)) ->
       Format.fprintf ppf "Collision on label hash: {%a}:%a, {%a}:%a" 
