@@ -1,19 +1,20 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
 #load "q_MLast.cmo";;
-
 (* TODO:
    - optimizations: generate labels and atoms only once.
    - translate record to open record on positive occurence
 *)
 
-
 open Mltypes
 open Ident
+open Camlp4.PreCast
+
+let _loc = Loc.ghost
 
 module IntMap = 
   Map.Make(struct type t = int let compare : t -> t -> int = compare end)
@@ -31,6 +32,17 @@ let atom lab = Types.atom (Atoms.atom (Atoms.V.mk_ascii lab))
 let label lab = Label.mk (Ns.empty, U.mk lab)
 let bigcup f l = List.fold_left (fun accu x -> Types.cup accu (f x)) Types.empty l
 
+let id s =
+  let rec aux i : Ast.ident =
+    try
+      let j = String.index_from s i '.' in
+      <:ident< $uid:String.sub s i (j - i)$.$aux (j+1)$ >>
+    with Not_found ->
+      <:ident< $lid:String.sub s i (String.length s - i)$ >>
+  in
+(*  Printf.eprintf "*** %S\n" s; *)
+  aux 0
+
 let rec typ t =
   try IntHash.find memo_typ t.uid
   with Not_found ->
@@ -83,11 +95,6 @@ let mk_var _ =
 
 let mk_vars = List.map mk_var
 
-let _loc = (Lexing.dummy_pos,Lexing.dummy_pos)
-
-let let_in p e body =
-  <:expr< let $list:[ p, e ]$ in $body$ >>
-
 let atom_ascii lab =
   <:expr< Value.atom_ascii $str: String.escaped lab$ >>
 
@@ -97,16 +104,15 @@ let label_ascii lab =
 let pair e1 e2 = <:expr< Value.Pair ($e1$,$e2$) >>
 
 let pmatch e l = 
-  let l = List.map (fun (p,e) -> p,None,e) l in
   <:expr< match $e$ with [ $list:l$ ] >>
 
 let rec matches ine oute = function
   | [v1;v2] ->
-      let_in <:patt<($lid:v1$,$lid:v2$)>> <:expr< Value.get_pair $ine$ >> oute
+      <:expr< let ($lid:v1$,$lid:v2$) = Value.get_pair $ine$ in $oute$ >>
   | v::vl ->
       let r = mk_var () in
       let oute = matches <:expr< $lid:r$ >> oute vl in
-      let_in <:patt<($lid:v$,$lid:r$)>> <:expr< Value.get_pair $ine$ >> oute
+      <:expr< let ($lid:v$,$lid:r$) = Value.get_pair $ine$ in $oute$ >>
   | [] -> assert false
 
 let list_lit el =
@@ -175,7 +181,7 @@ let rec tuple = function
 
 let pat_tuple vars = 
   let pl = List.map (fun id -> <:patt< $lid:id$ >>) vars in
-  <:patt< ($list:pl$) >>
+  <:patt< ($Ast.paCom_of_list pl$) >>
 
 
 let call_lab f l x =
@@ -202,7 +208,7 @@ let rec to_cd e t =
 (*  Format.fprintf Format.err_formatter "to_cd %a [uid=%i; recurs=%i]@."
     Mltypes.print t t.uid t.recurs;  *)
   if t.recurs > 0 then <:expr< $lid:to_cd_fun t$ $e$ >>
-  else to_cd_descr e t.def
+  else to_cd_descr e t.def 
 
 and to_cd_descr e = function
   | Link t -> to_cd e t
@@ -225,7 +231,7 @@ and to_cd_descr e = function
   | Tuple tl -> 
       (* let (x1,...,xn) = ... in Value.Pair (t1(x1), Value.Pair(...,tn(xn))) *)
       let vars = mk_vars tl in
-      let_in (pat_tuple vars) e (tuple (tuple_to_cd tl vars))
+      <:expr< let $pat_tuple vars$ = $e$ in $tuple (tuple_to_cd tl vars)$ >>
   | PVariant l ->
       (* match <...> with 
 	 | `A -> Value.atom_ascii "A" 
@@ -234,10 +240,9 @@ and to_cd_descr e = function
       let cases = 
 	List.map
 	  (function 
-	     | (lab,None) -> <:patt< `$lid:lab$ >>, atom_ascii lab
-	     | (lab,Some t) -> 
-		 <:patt< `$lid:lab$ x >>, 
-		 pair (atom_ascii lab) (to_cd <:expr< x >> t)
+	     | (lab,None) -> <:match_case< `$lid:lab$ -> $atom_ascii lab$ >>
+	     | (lab,Some t) -> <:match_case< `$lid:lab$ x ->
+		 $pair (atom_ascii lab) (to_cd <:expr< x >> t)$ >>
 	  ) l in
       pmatch e cases
   | Variant (p,l,_) ->
@@ -252,13 +257,13 @@ and to_cd_descr e = function
 		 let pat = match lab with (* Stupid Camlp4 *)
 		    | "true" -> <:patt< True >>
 		    | "false" -> <:patt< False >>
-		    | lab -> <:patt< $lid:p^lab$ >>
+		    | lab -> <:patt< $id: id (p^lab)$ >>
 		 in
-		 pat, atom_ascii lab
+		 <:match_case< $pat$ -> $atom_ascii lab$ >>
 	     | (lab,tl) -> 
 		 let vars = mk_vars tl in
-		 <:patt< $lid:p^lab$ $pat_tuple vars$ >>,
-		 tuple (atom_ascii lab :: tuple_to_cd tl vars)
+		 <:match_case< $id: id (p^lab)$ $pat_tuple vars$ ->
+	           $tuple (atom_ascii lab :: tuple_to_cd tl vars)$ >>
 	  ) l in
       pmatch e cases
   | Record (p,l,_) ->
@@ -268,7 +273,7 @@ and to_cd_descr e = function
 	 let l = 
 	   List.map
 	     (fun (lab,t) ->
-		let e = to_cd <:expr<$x$.$lid:p^lab$>> t in
+		let e = to_cd <:expr<$x$.$id:id (p^lab)$>> t in
 		<:expr< ($label_ascii lab$, $e$) >>)
 	     l
 	 in
@@ -320,7 +325,7 @@ and tuple_to_cd tl vars = List.map2 (fun t id -> to_cd <:expr< $lid:id$ >> t) tl
 
 
 
-and to_ml e t =
+and to_ml (e : Ast.expr) (t : Mltypes.t) =
 (*  Format.fprintf Format.err_formatter "to_ml %a@."
     Mltypes.print t;  *)
   if t.recurs > 0 then <:expr< $lid:to_ml_fun t$ $e$ >>
@@ -346,8 +351,7 @@ and to_ml_descr e = function
 	 (t1(x1),...,tn(xn)) *)
 
       let vars = mk_vars tl in
-      let el = tuple_to_ml tl vars in
-      matches e <:expr< ($list:el$) >> vars
+      matches e <:expr< $tuple_to_ml tl vars$ >> vars
   | PVariant l ->
       (* match Value.get_variant <...> with 
 	 | "A",None -> `A 
@@ -358,15 +362,16 @@ and to_ml_descr e = function
 	List.map 
 	  (function 
 	     | (lab,None) -> 
-		 <:patt< ($str: String.escaped lab$, None) >>,
-		 <:expr< `$lid:lab$ >>
+		 <:match_case< 
+		   ($str: String.escaped lab$, None) -> `$lid:lab$ >>
 	     | (lab,Some t) ->
 		 let x = mk_var () in
 		 let ex = <:expr< $lid:x$ >> in
-		 <:patt< ($str: String.escaped lab$, Some $lid:x$) >>,
-		 <:expr< `$lid:lab$ $to_ml ex t$ >>
+		 <:match_case< 
+		   ($str: String.escaped lab$, Some $lid:x$) ->
+	           `$lid:lab$ $to_ml ex t$ >>
 	  ) l in
-      let cases = cases @ [ <:patt< _ >>, <:expr< assert False >> ] in
+      let cases = cases @ [ <:match_case< _ -> assert False >> ] in
       pmatch <:expr< Value.get_variant $e$ >> cases
   | Variant (_,l,false) ->
       failwith "Private Sum type"
@@ -379,25 +384,26 @@ and to_ml_descr e = function
 	List.map 
 	  (function 
 	     | (lab,[]) -> 
-		 <:patt< ($str: String.escaped lab$, None) >>,
-		 (match lab with (* Stupid Camlp4 *)
+		 let pa = <:patt< ($str: String.escaped lab$, None) >>
+		 and e = match lab with (* Stupid Camlp4 *)
 		    | "true" -> <:expr< True >>
 		    | "false" -> <:expr< False >>
-		    | lab -> <:expr< $lid:p^lab$ >>)
+		    | lab -> <:expr< $id:id (p ^ lab)$ >> in
+		 <:match_case< $pa$ -> $e$ >>
 	     | (lab,[t]) ->
 		 let x = mk_var () in
 		 let ex = <:expr< $lid:x$ >> in
-		 <:patt< ($str: String.escaped lab$, Some $lid:x$) >>,
-		 <:expr< $lid:p^lab$ $to_ml ex t$ >>
+		 <:match_case< ($str: String.escaped lab$, Some $lid:x$) ->
+		  $id:id (p ^ lab)$ $to_ml ex t$ >>
 	     | (lab,tl) ->
 		 let vars = mk_vars tl in
-		 let el = tuple_to_ml tl vars in
 		 let x = mk_var () in
-		 <:patt< ($str: String.escaped lab$, Some $lid:x$) >>,
-		 matches <:expr< $lid:x$ >> 
-		         <:expr< $lid:p^lab$ ($list:el$) >> vars
+		 <:match_case< ($str: String.escaped lab$, Some $lid:x$) ->
+		 $matches <:expr< $lid:x$ >> 
+		         <:expr< $id:id (p ^ lab)$ $tuple_to_ml tl vars$ >> 
+		   vars$ >>
 	  ) l in
-      let cases = cases @ [ <:patt< _ >>, <:expr< assert False >> ] in
+      let cases = cases @ [ <:match_case< _ -> assert False >> ] in
       pmatch <:expr< Value.get_variant $e$ >> cases
   | Record (_,l,false) ->
       failwith "Private Record type"
@@ -409,9 +415,9 @@ and to_ml_descr e = function
 	 let l = 
 	   List.map
 	     (fun (lab,t) ->
-		(<:patt< $lid:p^lab$>>,
-		 to_ml 
-		 <:expr< Value.get_field $x$ $label_ascii lab$ >> t)) l in
+		let e = 
+		  to_ml <:expr< Value.get_field $x$ $label_ascii lab$ >> t in
+		<:rec_binding< $id: id (p^lab)$ = $e$ >>) l in
 	 <:expr< {$list:l$} >>)
 
   | Abstract "int" -> <:expr< Value.cduce2ocaml_int $e$ >>
@@ -442,7 +448,9 @@ and to_ml_descr e = function
   | Var _ -> e
   | _ -> assert false
 
-and tuple_to_ml tl vars = List.map2 (fun t id -> to_ml <:expr< $lid:id$ >> t) tl vars
+and tuple_to_ml tl vars =
+  Ast.exCom_of_list 
+    (List.map2 (fun t id -> to_ml <:expr< $lid:id$ >> t) tl vars)
 
 
 let to_ml_done = IntHash.create 13
@@ -456,7 +464,7 @@ let global_transl () =
       IntHash.add don hd.uid ();
       let p = <:patt< $lid:fun_name hd$ >> in
       let e = <:expr< fun x -> $to_descr <:expr< x >> hd.def$ >> in
-      defs := (p,e) :: !defs
+      defs := <:binding< $p$ = $e$ >> :: !defs
     );
     loop ()
   and loop () = match !to_cd_gen,!to_ml_gen with
@@ -506,13 +514,25 @@ let check_value ty_env c_env (s,caml_t,t) =
   let x = mk_var () in
   let slot = Compile.find_slot id c_env in
   let e = to_ml <:expr< slots.($int:string_of_int slot$) >> t in
-  <:patt< $uid:s$ >>, <:expr< C.$uid:x$ >>, (<:patt< $uid:x$ >>, e)
+  <:patt< $lid:s$ >>, <:expr< C.$lid:x$ >>, <:binding< $lid:x$ = $e$ >>
+
+module Cleaner = Camlp4.Struct.CleanAst.Make(Ast)
+
+let cleaner = object
+  inherit Cleaner.clean_ast as super
+  method str_item st =
+    match super#str_item st with
+      | <:str_item< value $rec:_$ $ <:binding< >> $ >> ->
+	<:str_item< >>
+      | x -> x
+end
+
 
 let stub ty_env c_env exts values mk prolog =
   gen_types := false;
   let items = List.map (check_value ty_env c_env) values in
 
-  let exts = List.rev_map (fun (s,t) -> to_cd <:expr< $lid:s$ >> t) exts in
+  let exts = List.rev_map (fun (s,t) -> to_cd <:expr< $id:id s$ >> t) exts in
   let g = global_transl () in
 
   let types = get_registered_types () in
@@ -522,31 +542,23 @@ let stub ty_env c_env exts values mk prolog =
   let items_expr = List.map (fun (_,e,_) -> e)  items in
   let items_pat = List.map (fun (p,_,_) -> p) items in
 
-  let m = 
-    [ <:str_item< open Cduce_lib >>;
-      <:str_item< Config.init_all () >>;
-      <:str_item< 
-	value (types,set_externals,slots,run) = 
-	Librarian.ocaml_stub $str:String.escaped raw$ >> ] @
-      (if g = [] then [] else [ <:str_item< value rec $list:g$ >> ]) @    
-      [ <:str_item< set_externals [|$list:exts$|] >>;
-	<:str_item< run () >> ] @
-      (if items = [] then [] else [ <:str_item< value $list:items_def$ >> ]) in
-
-  let items_expr = 
-    match items_expr with 
-      | [] -> <:expr< () >> 
-      | l -> <:expr< ($list:l$) >> in
-
   let str_items = 
-    [ <:str_item< 
-      value ($list:items_pat$) = 
-	let module C = struct 
-	  $list:m$ 
-	end in $items_expr$ >>, (Lexing.dummy_pos, Lexing.dummy_pos) ] in
-
+    <:str_item< 
+      value $tup:Ast.paCom_of_list items_pat$ = 
+        let module C = struct
+	  open Cduce_lib;
+	  Cduce_config.init_all ();
+	  value (types,set_externals,slots,run) = 
+	    Librarian.ocaml_stub $str:String.escaped raw$;
+	  value rec $Ast.biAnd_of_list g$;
+	  set_externals [|$Ast.exSem_of_list exts$|]; 
+	  run ();
+	  value $Ast.biAnd_of_list items_def$; 
+	end in $tup:Ast.exCom_of_list items_expr$ >> in
+  
   print_endline prolog;
-  !Pcaml.print_implem str_items
+  try Printers.OCaml.print_implem (cleaner # str_item str_items)
+  with exn -> Format.printf "@."; raise exn
 (*  let exe = Filename.concat (Filename.dirname Sys.argv.(0)) "cdo2ml" in
   let oc = Unix.open_process_out exe in
   Marshal.to_channel oc str_items [];
@@ -567,7 +579,7 @@ let stub_ml name ty_env c_env exts mk =
       try Mltypes.read_cmi name
       with Not_found ->  ("",[]) in
     stub ty_env c_env exts values mk prolog
-  with Mltypes.Error s -> raise (Location.Generic s)
+  with Mltypes.Error s -> raise (Cduce_loc.Generic s)
 
 
 let register b s args = 
@@ -575,7 +587,7 @@ let register b s args =
     let (t,n) = Mltypes.find_value s in
     let m = List.length args in
     if n <> m then
-      Location.raise_generic
+      Cduce_loc.raise_generic
 	(Printf.sprintf 
 	   "Wrong arity for external symbol %s (real arity = %i; given = %i)" s n m);
     let i = if b then
@@ -590,7 +602,7 @@ let register b s args =
     vars := [| |];
     i,cdt
   with Not_found -> 
-    Location.raise_generic
+    Cduce_loc.raise_generic
       (Printf.sprintf "Cannot resolve ocaml external %s" s)
 
 (* Generation of wrappers *)
@@ -605,12 +617,12 @@ let wrapper values =
     values in
   let g = global_transl () in
 
-  let m = if g = [] then exts else <:str_item< value rec $list:g$ >>::exts in
-  let m = [ <:str_item< open Cduce_lib >>;
-	    <:str_item< Config.init_all () >>] @ m in
-
-  <:str_item< declare $list:m$ end >>
-
+  <:str_item< 
+    open Cduce_lib; 
+    Cduce_config.init_all ();
+    value rec $Ast.biAnd_of_list g$;
+    $Ast.stSem_of_list exts$;
+  >>
 
 let gen_wrapper vals =
   try
@@ -627,7 +639,7 @@ let gen_wrapper vals =
       ) [] vals in
 
     wrapper values
-  with Mltypes.Error s -> raise (Location.Generic s)
+  with Mltypes.Error s -> raise (Cduce_loc.Generic s)
 
 let make_wrapper fn = 
   let ic = open_in fn in
@@ -642,9 +654,9 @@ let make_wrapper fn =
    done 
    with End_of_file -> ());
   let s = gen_wrapper !v in
-  !Pcaml.print_implem [ s,_loc ];
-  print_endline "let () = Location.obj_path := [";
-  List.iter (fun s -> Printf.printf "  %S;\n" s) !Location.obj_path;
+  Printers.OCaml.print_implem s;
+  print_endline "let () = Cduce_loc.obj_path := [";
+  List.iter (fun s -> Printf.printf "  %S;\n" s) !Cduce_loc.obj_path;
   print_endline " ];;";
   print_endline "let () = Run.main ();;"
 
@@ -684,7 +696,7 @@ let register () =
   Librarian.make_wrapper := make_wrapper
 
 let () =
-  Config.register 
+  Cduce_config.register 
     "ocaml" 
     "OCaml interface" 
     register
