@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
 let error msg =
@@ -35,7 +35,7 @@ let load_url s =
 	error msg
 
 let () = 
-  Config.register 
+  Cduce_config.register 
     "netclient" 
     "Load external URLs with netclient"
     (fun () -> Url.url_loader := load_url)
