@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
 let start_with s p =
@@ -20,7 +20,7 @@ let no_load_url s =
     Printf.sprintf 
       "Error \"%s\": \nTo fetch external URLs, you need to compile CDuce with curl and/or netclient" s 
   in
-  raise (Location.Generic msg) 
+  raise (Cduce_loc.Generic msg) 
 
 let url_loader = ref no_load_url
 
