@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
 (* Loading XML documents *)
@@ -124,7 +124,7 @@ let load_html s =
     | h::t -> val_of_doc (val_of_docs t) h
   in	
 
-  Location.protect_op "load_html";
+  Cduce_loc.protect_op "load_html";
   let parse src = Nethtml.parse_document ~dtd:Nethtml.relaxed_html40_dtd src in
   let doc = 
     if Url.is_url s then
