@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
 open Ident
@@ -270,7 +270,7 @@ let rec is_str = function
 let rec print ppf v =
   if is_str v then 
     (Format.fprintf ppf "\"";
-     print_quoted_str ppf v;
+     ignore (print_quoted_str ppf v);
      Format.fprintf ppf "\"")
   else if is_seq v then Format.fprintf ppf "[ @[<hv>%a@]]" print_seq v
   else  match v with
