@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
 open Ident
@@ -1292,7 +1292,9 @@ struct
 
   let empty_cases d =
     let x = BoolRec.compute
-	      ~empty:0 ~full:3 ~cup:(lor) ~cap:(land)
+	      ~empty:0 ~full:3 
+	      ~cup:(fun x y -> x lor y) 
+	      ~cap:(fun x y -> x land y)
 	      ~diff:(fun a b -> a land lnot b)
 	      ~atom:(function (o,r) ->
 		       assert (LabelMap.get r == []);
