@@ -1,10 +1,10 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
 (**************************************************************************)
 
-open Location
+open Cduce_loc
 open Ast
 open Ident
 
@@ -15,10 +15,10 @@ let (>=) (x:int) y = x >= y
 let (>) (x:int) y = x > y
 
 let warning loc msg =
-  let v = Location.get_viewport () in
+  let v = Cduce_loc.get_viewport () in
   let ppf = if Html.is_html v then Html.ppf v else Format.err_formatter in
-  Location.print_loc ppf (loc,`Full);
-  Location.html_hilight (loc,`Full);
+  Cduce_loc.print_loc ppf (loc,`Full);
+  Cduce_loc.html_hilight (loc,`Full);
   Format.fprintf ppf "Warning: %s@." msg
 
 exception NonExhaustive of Types.descr
@@ -815,7 +815,7 @@ open Typed
 let localize loc f x =
   try f x
   with 
-    | (Error _ | Constraint (_,_)) as exn -> raise (Location.Location (loc,`Full,exn))
+    | (Error _ | Constraint (_,_)) as exn -> raise (Cduce_loc.Location (loc,`Full,exn))
     | Warning (s,t) -> warning loc s; t
 
 let require loc t s = 
@@ -968,7 +968,7 @@ and type_check' loc env e constr precise = match e with
 	    ) t
 	with (Sequence.Error _) as exn -> 
 	  let rec find_loc = function
-	    | Location.Location (loc,precise,exn) ->
+	    | Cduce_loc.Location (loc,precise,exn) ->
 		(loc,precise), exn
 	    | Sequence.Error (Sequence.UnderTag (t,exn)) ->
 		let (l,exn) = find_loc exn in
@@ -977,7 +977,7 @@ and type_check' loc env e constr precise = match e with
 	  in
 	  try 
 	    let (loc,precise), exn = find_loc exn in
-	    raise (Location.Location (loc,precise,exn))
+	    raise (Cduce_loc.Location (loc,precise,exn))
 	  with Not_found ->
 	    raise_loc loc exn
       in
